package com.yeejoin.amos.iot.business.controller;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolationException;

import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.transaction.TransactionSystemException;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yeejoin.amos.iot.business.param.CompanyBo;
import com.yeejoin.amos.iot.business.param.ReginParams;
import com.yeejoin.amos.iot.context.RedisUtil;
import com.yeejoin.amos.iot.core.common.request.CommonRequest;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.iot.core.util.StringUtil;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.util.ApplicationConfig;
import com.yeejoin.amos.security.common.context.CurrentAuthentication;
import com.yeejoin.amos.security.common.model.UserModel;
import com.yeejoin.amos.security.common.remote.RemoteSecurityServer;

import springfox.documentation.annotations.ApiIgnore;

/**
 * 基础控制器
 * 
 * @author as-youjun
 *
 */
@ApiIgnore
@RestController
@RequestMapping("/base")
@com.yeejoin.amos.security.common.authorization.Authorization
public class BaseController {
	
	@Autowired
	protected RedisUtil redisUtil;
	
	@Autowired
	private RemoteSecurityServer remoteSecurityServer;
	
	@Autowired
	protected HttpServletRequest request;
	
	@Autowired
	private ApplicationConfig applicationConfig;
	private  RestTemplate restTemplate;
	private static String X_ACCESS_TOKEN = null;
	/**
	 * 成功返回状态
	 */
	protected static final String SUCCESS = "SUCCESS";

	/**
	 * 失败返回状态
	 */
	protected static final String FAILED = "FAILED";

	/**
	 * 默认页大小
	 */
	protected static final int DEFAULT_PAGE_SIZE = 10;
	

	protected String getToken() 
	{
		/* Get token from header */
		String authToken = request.getHeader("X-Access-Token");
		/* If token not found get it from request parameter */
		if (authToken == null)
		{
			authToken = request.getParameter("token");
		}
		return authToken;
	}
	
	/**
	 * 
	 * <pre>
	 * 判断当前用户角色级别：true:主管/false:装备管理员
	 * </pre>
	 *
	 * @return
	 */
//	protected boolean isDirector()
//	{
//		if(getUserInfo() == null){
//			return false;
//		}
//		return getUserInfo().getRoles().get(0).get("");
//	}
//	
	/**
	 * 当前登录用户信息
	 */
//	protected User getUserInfo() {
//		if(getToken() == null){
//			return null;
//		}
//		return remoteSecurityServer.loginAuthentication(getToken());
//	}
	
	/**
	 * 当前登录用户信息
	 */
//	protected UserModel getUserInfo() {
//		if(getToken() == null){
//			return null;
//		}
//		return remoteSecurityServer.loginAuthentication(getToken());
//	}
	
	//新安全
	protected UserModel getUserInfo() {
		UserModel  user=CurrentAuthentication.getAuthenticatedUserThreadLocal();
		return user;
	}
	
	
	
	/**
	 * 获取当前用户所属公司名称
	 * @return
	 */
//	protected String getCurrentCompanyNameOfUser(){
//		if(getUserInfo() == null){
//			return "";
//		}
//		return getUserInfo().getCompany().getCompanyName();
//	}
	
	protected String getCurrentCompanyNameOfUser(){
		ReginParams reginParams = this.getSelectedOrgInfo();
		if(reginParams == null || reginParams.getCompany() == null){
			return "";
		}
		return reginParams.getCompany().getCompanyName();
	}
	
	protected String getUserId() 
	{
		String userId = null;
		if (getUserInfo() != null)
		{
			userId = getUserInfo().getUserId();
		}
		return userId;
	}
	
	/**
	 * 获取公司orgcode
	 * @return
	 */
//	protected String getCompanyOrgCode(){
//		if(getUserInfo() == null){
//			return "";
//		}
//		return getUserInfo().getCompany().getCompCode(); 
//	}
	protected String getOrgCode(){
		ReginParams reginParams = this.getSelectedOrgInfo();
		if(reginParams == null){
			return "";
		}
		if (reginParams.getDepartment() == null || reginParams.getDepartment().getOrgCode() == null) {
			return reginParams.getCompany().getOrgCode();
		}

		return reginParams.getDepartment().getOrgCode();
	}

	protected String getCompanyOrgCode(){
		ReginParams reginParams = this.getSelectedOrgInfo();
		if(reginParams == null){
			return "";
		}
		return reginParams.getCompany().getOrgCode();
	}

	/**
	 * 
	 * <pre>
	 * 生成查询条件
	 * </pre>
	 * 
	 * @param queryRequests
	 * @return
	 */
	public List<DaoCriteria> buildDaoCriterias(List<CommonRequest> queryRequests) {
		List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
		String orgCode = null;
		Boolean flag = true;
		if(queryRequests != null && !queryRequests.isEmpty()){
			for (CommonRequest query : queryRequests) {
				DaoCriteria criteria = new DaoCriteria();
				if(StringUtil.isNotEmpty(query.getValue())){
					if(query.getName().equals("orgCode")){
						orgCode = query.getValue().toString();
						continue;
					}
					
					if("visibility".equals(query.getName()) && "0".equals(query.getValue())){
						flag = false;
					}
					criteria.setPropertyName(query.getName());
					String column = criteria.getPropertyName();
					if (!(query.getValue() instanceof Collection<?>) 
							&& column.substring(column.length()-2,column.length()).toUpperCase().equals("ID")) {
						criteria.setValue(Long.valueOf(query.getValue().toString()));
					} else {
						criteria.setValue(query.getValue());
					}
					
					String operator = query.getType();
					if (!StringUtil.isNotEmpty(operator)) {
						criteria.setOperator(QueryOperatorEnum.EQUAL.getName());
					}else if(operator.equals(QueryOperatorEnum.LIKE.getName())){
						setLikeCriteria(criteria,query);
					}else if(operator.equals(QueryOperatorEnum.BIGGER_EQUAL.getName())){
						criteria.setOperator(QueryOperatorEnum.getEnum(operator).getName());
					}else if(operator.equals(QueryOperatorEnum.LESS_EQUAL.getName())){
						criteria.setOperator(QueryOperatorEnum.getEnum(operator).getName());
					}else {
						criteria.setOperator(QueryOperatorEnum.getEnum(operator)
								.getName());
					}
					daoCriterias.add(criteria);
				}
			}	
		}
		if(flag){
			
			//获取当前用户code			
			orgCode=getCompanyOrgCode();
			daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias,orgCode);
//			if(isDirector()){
//				daoCriterias = buildOrgDaoCriteriaInChildren(daoCriterias,orgCode);
//			}else{
//				daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias);
//			}
		}
		daoCriterias.add(new DaoCriteria("isDelete", QueryOperatorEnum.EQUAL.getName(), false));
		return daoCriterias;

	}
	
	public List<DaoCriteria> buildDaoCriteriasNoDelete(List<CommonRequest> queryRequests) {
		List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
		String orgCode = null;
		Boolean flag = true;
		if(queryRequests != null && !queryRequests.isEmpty()){
			for (CommonRequest query : queryRequests) {
				DaoCriteria criteria = new DaoCriteria();
				if(StringUtil.isNotEmpty(query.getValue())){
					if(query.getName().equals("orgCode")){
						orgCode = query.getValue().toString();
						continue;
					}
					if("visibility".equals(query.getName()) && "0".equals(query.getValue())){
						flag = false;
					}
					criteria.setPropertyName(query.getName());
					String column = criteria.getPropertyName();
					if (!(query.getValue() instanceof Collection<?>) 
							&& column.substring(column.length()-2,column.length()).toUpperCase().equals("ID")) {
						criteria.setValue(Long.valueOf(query.getValue().toString()));
					} else {
						criteria.setValue(query.getValue());
					}
					
					String operator = query.getType();
					if (!StringUtil.isNotEmpty(operator)) {
						criteria.setOperator(QueryOperatorEnum.EQUAL.getName());
					}else if(operator.equals(QueryOperatorEnum.LIKE.getName())){
						setLikeCriteria(criteria,query);
					}else if(operator.equals(QueryOperatorEnum.BIGGER_EQUAL.getName())){
						criteria.setOperator(QueryOperatorEnum.getEnum(operator).getName());
					}else if(operator.equals(QueryOperatorEnum.LESS_EQUAL.getName())){
						criteria.setOperator(QueryOperatorEnum.getEnum(operator).getName());
					}else {
						criteria.setOperator(QueryOperatorEnum.getEnum(operator)
								.getName());
					}
					daoCriterias.add(criteria);
				}
			}	
		}
		if(flag){
			daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias,orgCode);
//			if(isDirector()){
//				daoCriterias = buildOrgDaoCriteriaInChildren(daoCriterias,orgCode);
//			}else{
//				daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias);
//			}
		}
		return daoCriterias;

	}
	
	public List<DaoCriteria> buildDaoCriteriasNoParam() {
		List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
		String orgCode = getCompanyOrgCode();
		daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias,orgCode);
//		if(isDirector()){
//			daoCriterias = buildOrgDaoCriteriaInChildren(daoCriterias,orgCode);
//		}else{
//			daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias);
//		}
		return daoCriterias;
	}
	
	/**
	 * 
	 * <pre>
	 * 生成通过当前用户所属组织机构（包含子机构）过滤的查询条件
	 * </pre>
	 *
	 * @param daoCriterias 查询条件集合
	 * @return
	 */
//	public List<DaoCriteria> buildOrgDaoCriteriaInChildren(List<DaoCriteria> daoCriterias,String orgCode) 
//	{
//		/**
//		 * 当前用户所属公司
//		 */
//		String companyCode = orgCode;
//		if(companyCode == null){
//			CompanyBo company = getUserInfo().getCompany();
//			companyCode = company.getCompCode();
//		}
//		
//		//添加所属公司的过滤条件
//		DaoCriteria compDaoCriteria = new DaoCriteria();
//		compDaoCriteria.setPropertyName("orgCode");
//		compDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
//		compDaoCriteria.setValue(companyCode);
//		daoCriterias.add(compDaoCriteria);
//		
//		
//		//添加子公司下的过滤条件
//		DaoCriteria childCompDaoCriteria = new DaoCriteria();
//		childCompDaoCriteria.setPropertyName("orgCode");
//		childCompDaoCriteria.setOperator(QueryOperatorEnum.LIKE.getName());
//		childCompDaoCriteria.setValue(companyCode+"*%");
//		daoCriterias.add(childCompDaoCriteria);
//		return daoCriterias;
//	}
	
	/**
	 * 
	 * <pre>
	 * 生成通过当前用户所属组织机构（不包含子机构）过滤的查询条件
	 * </pre>
	 *
	 * @param daoCriterias 查询条件集合
	 * @return
	 */
	public List<DaoCriteria> buildOrgDaoCriteriaOutChildren(List<DaoCriteria> daoCriterias, String orgCode) 
	{
		/**
		 * 当前用户所属公司
		 */
		
		DaoCriteria compDaoCriteria = new DaoCriteria();
		compDaoCriteria.setPropertyName("orgCode");
		compDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
		compDaoCriteria.setValue(orgCode);
		daoCriterias.add(compDaoCriteria);
		return daoCriterias;
	}

	/**
	 * 对like查询语句的内容进行特殊字符转义
	 * @param criteria
	 * @param query
	 */
	@SuppressWarnings("unchecked")
	public void setLikeCriteria(DaoCriteria criteria,CommonRequest query){
		if (query.getValue() instanceof List) {
			List<String> value = (List<String>) query.getValue();
			if (!value.isEmpty()) {
				value = value.stream().map(s -> {
					String str = s.replace("%","/%").replace("_","/_");
					return "%" + str + "%";
				}).collect(Collectors.toList());
				criteria.setValue(value);
				criteria.setOperator(QueryOperatorEnum.LIKE.getName());
			}
		} else {
			String value = (String) query.getValue();
			if(value != null){
				value = value.replace("%","/%").replace("_","/_");
				criteria.setValue("%"+value+"%");
				criteria.setOperator(QueryOperatorEnum.LIKE.getName());
			}
		}
	}
	
	/**
	 * <pre>
	 * 提取错误异常中的错误消息
	 * </pre>
	 * 
	 * <p>
	 * 当实体类中对属相注解了以下类似的注解，需要用try.catch语句捕获异常，使用 #getErrorMessage(Exception)
	 * 提取出异常信息NotNull(message = "属性不能为空！")
	 * </p>
	 * @see {@code NotBlank} ... NotNull、NotEmpty...
	 * @see {@code Valid}
	 * 
	 * @param e
	 * @return
	 */
	public String getErrorMessage(Exception e){
		StringBuilder message = new StringBuilder();
		if(e instanceof TransactionSystemException){
			TransactionSystemException exception = (TransactionSystemException) e;
			if(exception.getRootCause() instanceof ConstraintViolationException) {
				ConstraintViolationException root = (ConstraintViolationException) exception.getRootCause();
				root.getConstraintViolations().forEach(constraintViolation->{message.append(constraintViolation.getMessageTemplate());});
			}else{
				message.append(e.getMessage());
			}
		} else if (e instanceof ConstraintViolationException) {
			((ConstraintViolationException) e).getConstraintViolations().forEach(constraintViolation -> message.append(constraintViolation.getMessageTemplate()));
		} else {
			message.append("操作异常！");
		}
		return message.toString();
	}
	
	protected ReginParams getSelectedOrgInfo() {
		String json = (String) redisUtil.hget("regoin_"+getUserId(), getToken().substring(0, getToken().indexOf('_', 1)));
		return JSONObject.parseObject(json, ReginParams.class);
	}

	protected void saveSelectedOrgInfo(ReginParams reginParams) {
		redisUtil.hset("regoin_"+getUserId(), getToken().substring(0, getToken().indexOf('_', 1)), JSONObject.toJSONString(reginParams));
	}
}
