package com.yeejoin.amos.iot.business.service.impl;

import java.util.List;
import java.util.stream.Collectors;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.yeejoin.amos.iot.business.dao.repository.IRuleDao;
import com.yeejoin.amos.iot.business.entity.DroolsRule;
import com.yeejoin.amos.iot.business.entity.NodeVo;
import com.yeejoin.amos.iot.business.entity.Rule;
import com.yeejoin.amos.iot.business.entity.RuleGroup;
import com.yeejoin.amos.iot.business.param.NodeStyle;
import com.yeejoin.amos.iot.business.param.NodeVoParam;
import com.yeejoin.amos.iot.business.param.RuleDataParam;
import com.yeejoin.amos.iot.business.param.RulePageParam;
import com.yeejoin.amos.iot.business.param.TopoConfigParam;
import com.yeejoin.amos.iot.business.service.intfc.IDroolsRuleService;
import com.yeejoin.amos.iot.business.service.intfc.ILinkVoService;
import com.yeejoin.amos.iot.business.service.intfc.INodeVoService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleGroupService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleService;
import com.yeejoin.amos.iot.core.common.response.CommonPage;
import com.yeejoin.amos.iot.core.util.DateUtil;
import com.yeejoin.amos.iot.core.util.query.QuerySpecificationHelper;
import com.yeejoin.amos.iot.rule.engine.service.IRuleEngineService;
import com.yeejoin.amos.op.core.entity.handler.EntityChangeHandler;
import com.yeejoin.amos.op.core.jpa.GenericManagerImpl;
import com.yeejoin.amos.security.common.context.CurrentAuthentication;

/**
 * 
 * <pre>
 * 规则Service实现
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: RuleServiceImpl.java, v 0.1 2018年4月10日 下午3:55:30 as-chenjiajun Exp $
 */
@Service("ruleService")
public class RuleServiceImpl extends GenericManagerImpl<Rule, String> implements IRuleService
{
	@Autowired
	private ILinkVoService iLinkVoService;
	
	@Autowired
	private INodeVoService iNodeVoService;
	
	@Autowired
	private IRuleGroupService ruleGroupService;
	
	@Autowired
	private IRuleEngineService ruleEngineService;
	
	@Autowired
	private IDroolsRuleService droolsRuleService;
	
	IRuleDao ruleDao;
	
	public RuleServiceImpl(IRuleDao ruleDao)
	{
		super(ruleDao);
		this.ruleDao = ruleDao;
	}
	
	@Override
	public CommonPage<Rule> pageRuleData(RulePageParam request)
	{
		Pageable pageable = request.getSort() == null
				? new PageRequest(request.getPageNumber(),
						request.getPageSize())
				: new PageRequest(request.getPageNumber(),
						request.getPageSize(), request.getSort());
		try
		{			
			List<Rule> result = findAll(
					QuerySpecificationHelper.getSpecification(request));
			return new CommonPage<>(result, pageable,
					result.size());
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public RuleDataParam fetchRuleDataByID(String id)
	{
		RuleDataParam ruleDataParam = new RuleDataParam();
		Rule rule = findOne(id);
		ruleDataParam.setId(id);
		ruleDataParam.setRule(rule);
		ruleDataParam.setLinkVoList(iLinkVoService.findByRuleId(id));
		List<NodeVo> nodeVos = iNodeVoService.findByRuleId(id);
		ruleDataParam.setNodeVoList(nodeVos.stream().map(n -> new NodeVoParam(
				n.getId(),
				new TopoConfigParam(n.getDisplayName(),
						new NodeStyle(n.getX(), n.getY()), n.getType(),n.getNodeType())))
				.collect(Collectors.toList()));
		return ruleDataParam;
	}

	@Override
	public Rule saveRuleEntity(@Valid Rule rule)
	{
		rule.setCreateDate(DateUtil.getCurrentDate());
		rule.setCreateUser(CurrentAuthentication.getAuthenticatedUserId());
		rule = save(rule);
		return rule;
	}
	
	@Override
	public Rule editRuleEntity(@Valid Rule rule)
	{
		rule = save(rule);
		return rule;
	}

	@Transactional
	@Override
	public void startOrStopRule(String id)
	{
		Rule rule = findOne(id);
		if (rule != null)
		{
			rule.setIsUser(!rule.getIsUser());
			save(rule);
			//更新droolsRule中的isUser
			List<DroolsRule> droolsRuleList =droolsRuleService.findByRuleId(rule.getId());
			for (DroolsRule droolsRule : droolsRuleList)
			{
				droolsRule.setIsUser(rule.getIsUser());
			}
			droolsRuleService.save(droolsRuleList);
			//更新规则引擎
			RuleGroup ruleGroup = ruleGroupService.findOne("1");
			List<DroolsRule> droolsRules = droolsRuleService.findByGroupIdAndIsUser(ruleGroup.getId(), true);
			ruleEngineService.updateRule(ruleGroup, droolsRules);
		}
	}
	
	@Transactional
	@Override
	public void startOrStopRules(Boolean status, String orgCode)
	{
		RulePageParam request = new RulePageParam();
		request.setOrgCode(orgCode);
		List<Rule> rules = findAll(QuerySpecificationHelper.getSpecification(request));
		rules.forEach(rule -> {
			rule.setIsUser(status);
			save(rule);
			//更新droolsRule中的isUser
			List<DroolsRule> droolsRuleList =droolsRuleService.findByRuleId(rule.getId());
			for (DroolsRule droolsRule : droolsRuleList)
			{
				droolsRule.setIsUser(rule.getIsUser());
			}
			droolsRuleService.save(droolsRuleList);
		});
		//更新规则引擎
		RuleGroup ruleGroup = ruleGroupService.findOne("1");
		List<DroolsRule> droolsRules = droolsRuleService.findByGroupIdAndIsUser(ruleGroup.getId(), true);
		ruleEngineService.updateRule(ruleGroup, droolsRules);
	}

	@Override
	public Rule changeRuleStatus(Rule rule)
	{
		Rule persistRule = findOne(rule.getId());
		persistRule.setStatus(rule.getStatus());
		save(persistRule);
		return persistRule;
	}

	@Transactional
	@Override
	public void deleteRule(String id)
	{
		EntityChangeHandler.fireResourceReomoved(findOne(id));
		ruleDao.delRuleById(id);
	}
}
