package com.yeejoin.amos.iot.core.util;

import java.net.URISyntaxException;

import net.sf.json.JSONObject;

import org.apache.log4j.Logger;
import org.fusesource.mqtt.client.FutureConnection;
import org.fusesource.mqtt.client.MQTT;
import org.fusesource.mqtt.client.QoS;

/**
 * 
 * <pre>
 * Mqtt发送工具类
 * </pre>
 *
 * @author as-guowubin
 * @version $Id: MqttPublisher.java, v 0.1 2018年5月7日 下午2:21:01 as-guowubin Exp $
 */
public class MqttPublisher
{
	private static final Logger log = Logger.getLogger(MqttPublisher.class);
	private final static boolean CLEAN_START = true;
	private final static short KEEP_ALIVE = 30;// 低耗网络，但是又需要及时获取数据，心跳30s
	public final static long RECONNECTION_ATTEMPT_MAX = 6;
	public final static long RECONNECTION_DELAY = 2000;

	public final static int SEND_BUFFER_SIZE = 2 * 1024 * 1024;// 发送最大缓冲为2M

	/**
	 * 
	 * <pre>
	 * Mqtt消息发送
	 * </pre>
	 *
	 * @param user mqtt服务器用户名
	 * @param password mqtt服务器密码
	 * @param host mqtt服务器地址
	 * @param port mqtt服务器端口
	 * @param message 发送消息内容
	 * @param topic 消息主题
	 */
	public static void publisher(String user, String password, String host, 
			String port, Object message, String topic) throws Exception
	{
		MQTT mqtt = new MQTT();
		try {
			// 设置服务端的ip
			mqtt.setHost("tcp://" + host + ":" + port);
			mqtt.setUserName(user);
			mqtt.setPassword(password);
			// 连接前清空会话信息
			mqtt.setCleanSession(CLEAN_START);
			// 设置重新连接的次数
			mqtt.setReconnectAttemptsMax(RECONNECTION_ATTEMPT_MAX);
			// 设置重连的间隔时间
			mqtt.setReconnectDelay(RECONNECTION_DELAY);
			// 设置心跳时间
			mqtt.setKeepAlive(KEEP_ALIVE);
			// 设置缓冲的大小
			mqtt.setSendBufferSize(SEND_BUFFER_SIZE);

			// 创建连接
			final FutureConnection connection = mqtt.futureConnection();
			connection.connect();
			connection.publish(topic, JSONObject.fromObject(message).toString().getBytes("UTF-8"),
					QoS.AT_LEAST_ONCE, false);
		}
		catch (URISyntaxException e)
		{
			log.error("MqttAction执行失败：" + e.getMessage());
		}
		catch (Exception e)
		{
			log.error("MqttAction执行失败：" + e.getMessage());
		}
	}
}
